/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace ScreenQuad
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        Texture2D tex;
        BasicEffect fx;
        VertexBuffer vb;
        VertexDeclaration decl;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            this.IsMouseVisible = true;

            base.Initialize();

        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                tex = content.Load<Texture2D>(@".\Images\Test");
                // create the screen quad
                vb = CreateVB();
                // create the vertex decloration 
                decl = new VertexDeclaration(this.graphics.GraphicsDevice, VertexPositionNormalTexture.VertexElements);
                fx = new BasicEffect(this.graphics.GraphicsDevice, null);
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            if (Keyboard.GetState().IsKeyDown(Keys.Escape)) this.Exit();

            base.Update(gameTime);
        }

        void DrawQuad(Matrix view, Matrix proj, Matrix pos)
        {
            // setup base effect
            fx.View = view;
            fx.Projection = proj;
            fx.World = pos;
            fx.TextureEnabled = true;
            fx.Texture = tex;
            // set stream source
            this.graphics.GraphicsDevice.Vertices[0].SetSource(vb, 0, VertexPositionNormalTexture.SizeInBytes);
            
            // draw quad
            fx.Begin();
            foreach (EffectPass pass in fx.CurrentTechnique.Passes)
            {
                pass.Begin();
                this.graphics.GraphicsDevice.VertexDeclaration = decl;
                this.graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleFan, 0, 2);
                pass.End();
            }
            fx.End();
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // setup view and projection matrixes
            Matrix view = Matrix.CreateLookAt(Vector3.Forward, Vector3.Zero, Vector3.Up);
            float aspect = this.graphics.GraphicsDevice.Viewport.Width / this.graphics.GraphicsDevice.Viewport.Height;
            //Matrix proj = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver2, aspect, 1, 10000);
            Matrix proj = Matrix.CreateOrthographic(this.graphics.GraphicsDevice.Viewport.Width,
                                                    this.graphics.GraphicsDevice.Viewport.Height,
                                                    1,
                                                    10000);
            // this matrix stores models position/rotation/scale info etc
            // size the quad to the size of the active viewport
            Matrix pos = Matrix.CreateScale(this.graphics.GraphicsDevice.Viewport.Width,
                                      this.graphics.GraphicsDevice.Viewport.Height, 1);

            // now draw quad
            DrawQuad(view, proj, pos);

            base.Draw(gameTime);
        }

        /// <summary>
        /// Creates a triangle fan of 2 polys along the x/y axis centered around 0,0,0 with a width and height of 1.
        /// </summary>
        /// <returns>Returns a new vertexbuffer</returns>
        VertexBuffer CreateVB()
        {
            VertexPositionNormalTexture[] verts = new VertexPositionNormalTexture[4];
            verts[0] = new VertexPositionNormalTexture(new Vector3(-0.5f, -0.5f, 0),
                                                       new Vector3(-0.5f, -0.5f, 1),
                                                       new Vector2(1, 1));
            verts[1] = new VertexPositionNormalTexture(new Vector3(0.5f, -0.5f, 0),
                                                       new Vector3(0.5f, -0.5f, 1),
                                                       new Vector2(0, 1));
            verts[2] = new VertexPositionNormalTexture(new Vector3(0.5f, 0.5f, 0),
                                                       new Vector3(0.5f, 0.5f, 1),
                                                       new Vector2(0, 0));
            verts[3] = new VertexPositionNormalTexture(new Vector3(-0.5f, 0.5f, 0),
                                                       new Vector3(-0.5f, 0.5f, 1),
                                                       new Vector2(1, 0));

            VertexBuffer tmp = new VertexBuffer(this.graphics.GraphicsDevice, 4 * VertexPositionNormalTexture.SizeInBytes,
                                                ResourceUsage.None, ResourceManagementMode.Automatic);
            tmp.SetData<VertexPositionNormalTexture>(verts);
            return tmp;
        }
    }
}